package crystal.cfg;

import java.util.*;

import crystal.core.*;

/**
 *  Common base for all CFG Nodes --- provides a way of walking the tree.
 *  This class keeps track of the predecessor multiset, but the successor 
 *  lists are kept by subclasses, as they might have to annotate the edges. 
 */
public abstract class CFGNode {
	
	abstract public int numSuccs();
	abstract public CFGNode getSucc(int k);
	abstract void setSucc(int k, CFGNode n);
	
	List<CFGNode> preds = new ArrayList<CFGNode>(2);

	public int numPreds() {
		return preds.size();
	}
	
	public CFGNode getPred(int k) {
		return preds.get(k);
	}
	
	void addPred(CFGNode pred) {
		preds.add(pred);
	}

	abstract public void buildCoreExpressions();
	abstract public int numCoreKids();
	abstract public CoreExpr getCoreKid(int i);
	
	public boolean isEntry() {
		return preds.size() == 0;
	}

	public boolean isExit() {
		return numSuccs() == 0;
	}
	
	public boolean isSwitch() {
		return false;
	}

	public boolean isReturn() {
		return false;
	}
	
	public boolean isCall() {
		return false;
	}

	public boolean isAlloc() {
		return false;
	}

	public boolean isFree() {
		return false;
	}

	public boolean isAssign() {
		return false;
	}

	public CoreDerefExpr getWrittenMem() {
		return null;
	}

	public Set<CoreDerefExpr> getReadMem() {
		Set<CoreDerefExpr> set = new HashSet<CoreDerefExpr>();
		for (int i = 0; i < this.numCoreKids(); i++)
			set.addAll(getCoreKid(i).getMemExpr());
		return set;
	}

	protected void addRead(CoreExpr e, Set<CoreDerefExpr> set) {
		if (e instanceof CoreDerefExpr)
			set.add((CoreDerefExpr)e);
		
		for (int i = 0; i < e.numKids(); i++)
			addRead(e.getKid(i), set);
		
	}
	abstract public String toString();
}
